import { Injectable, ElementRef } from '@angular/core';
import { gsap } from 'gsap';
import { ScrollTrigger } from 'gsap/ScrollTrigger';
import { MotionPathPlugin } from 'gsap/MotionPathPlugin';

gsap.registerPlugin(ScrollTrigger, MotionPathPlugin);

@Injectable({
  providedIn: 'root',
})
export class AnimationService {
  private readonly MEGA_BONUS_THRESHOLD = 25000;
  private readonly BONUS_THRESHOLD = 1000;
  private readonly FLASH_THRESHOLD = 10000;

  private readonly ANIMATION_DURATIONS = {
    MEGA: 3,
    BONUS: 2,
    BASE: 1,
  };

  private readonly SYMBOLS = {
    MONEY: '💰',
    SPARKLE: '✨',
    GEM: '💎',
    STAR: '🌟',
  };

  constructor() {
    // Configure GSAP defaults
    gsap.config({
      autoSleep: 60,
      force3D: true,
      nullTargetWarn: false,
    });
  }

  animateEntrance(element: ElementRef, delay: number = 0) {
    return gsap.from(element.nativeElement, {
      duration: 0.6,
      opacity: 0,
      y: 30,
      ease: 'power3.out',
      delay,
      clearProps: 'all',
    });
  }

  animateFloat(element: ElementRef) {
    return gsap.to(element.nativeElement, {
      duration: 2,
      y: '-=20',
      ease: 'power1.inOut',
      yoyo: true,
      repeat: -1,
    });
  }

  animateShine(element: ElementRef) {
    const shine = gsap.to(element.nativeElement, {
      duration: 1.5,
      backgroundPosition: '200%',
      ease: 'linear',
      repeat: -1,
    });
    return shine;
  }

  animateMorphingBackground(element: ElementRef) {
    return gsap.to(element.nativeElement, {
      duration: 8,
      borderRadius: '60% 40% 30% 70% / 60% 30% 70% 40%',
      ease: 'sine.inOut',
      repeat: -1,
      yoyo: true,
    });
  }

  animateOnScroll(element: ElementRef, animation: 'fadeIn' | 'slideUp' | 'scaleIn' = 'fadeIn') {
    const animations = {
      fadeIn: {
        opacity: 0,
        y: 0,
        duration: 0.6,
      },
      slideUp: {
        opacity: 0,
        y: 50,
        duration: 0.8,
      },
      scaleIn: {
        opacity: 0,
        scale: 0.8,
        duration: 0.6,
      },
    };

    return gsap.from(element.nativeElement, {
      ...animations[animation],
      ease: 'power2.out',
      scrollTrigger: {
        trigger: element.nativeElement,
        start: 'top bottom-=100',
        toggleActions: 'play none none reverse',
      },
    });
  }

  createParticleEffect(container: ElementRef, particleCount: number = 20): void {
    const particles = Array.from({ length: particleCount }, () => this.createParticle(container));
    particles.forEach((particle) => this.animateParticle(particle));
  }

  private createParticle(container: ElementRef): HTMLElement {
    const particle = document.createElement('div');
    particle.className = 'absolute w-2 h-2 bg-emerald-500/20 rounded-full';
    container.nativeElement.appendChild(particle);

    gsap.set(particle, {
      x: gsap.utils.random(0, container.nativeElement.offsetWidth),
      y: gsap.utils.random(0, container.nativeElement.offsetHeight),
    });

    return particle;
  }

  private animateParticle(particle: HTMLElement): void {
    gsap.to(particle, {
      duration: gsap.utils.random(2, 4),
      x: '+=50',
      y: '-=50',
      opacity: 0,
      scale: 0,
      ease: 'none',
      repeat: -1,
      onRepeat: () => this.resetParticle(particle),
    });
  }

  private resetParticle(particle: HTMLElement): void {
    gsap.set(particle, {
      x: gsap.utils.random(0, particle.parentElement!.offsetWidth),
      y: gsap.utils.random(0, particle.parentElement!.offsetHeight),
      opacity: 1,
      scale: 1,
    });
  }

  animateButtonClick(element: ElementRef): gsap.core.Timeline {
    return gsap
      .timeline()
      .to(element.nativeElement, { scale: 0.95, duration: 0.1 })
      .to(element.nativeElement, { scale: 1, duration: 0.2, ease: 'elastic.out(1, 0.3)' });
  }

  animateSuccess(element: ElementRef): gsap.core.Timeline {
    return gsap
      .timeline()
      .to(element.nativeElement, { scale: 1.2, duration: 0.2, ease: 'power2.out' })
      .to(element.nativeElement, { scale: 1, duration: 0.5, ease: 'elastic.out(1, 0.3)' });
  }

  animateJackpotCounter(
    element: ElementRef,
    startValue: number,
    endValue: number
  ): gsap.core.Timeline {
    const container = this.prepareContainer(element);
    const increase = endValue - startValue;
    const timeline = gsap.timeline();

    if (increase > this.FLASH_THRESHOLD) {
      this.addFlashEffect(timeline, container, element);
    }

    this.addCounterAnimation(timeline, element, startValue, endValue);

    if (increase > this.MEGA_BONUS_THRESHOLD) {
      this.addMegaBonusEffect(element);
    } else if (increase > this.BONUS_THRESHOLD) {
      this.addBonusEffect(element);
    }

    return timeline;
  }

  private prepareContainer(element: ElementRef): HTMLElement {
    const container = element.nativeElement.parentElement;
    container.style.position = 'relative';
    this.cleanupExistingEffects(container);
    return container;
  }

  private cleanupExistingEffects(container: HTMLElement): void {
    const existingEffects = container.querySelectorAll('.jackpot-effect');
    existingEffects.forEach((effect: Element) => effect.remove());
  }

  private addFlashEffect(
    timeline: gsap.core.Timeline,
    container: HTMLElement,
    element: ElementRef
  ): void {
    const flash = this.createFlashElement();
    container.appendChild(flash);

    timeline
      .to(flash, {
        opacity: 1,
        duration: 0.3,
        yoyo: true,
        repeat: 2,
        onComplete: () => flash.remove(),
      })
      .to(
        element.nativeElement,
        {
          color: '#FFD700',
          textShadow: '0 0 20px rgba(255,215,0,0.8)',
          scale: 1.1,
          duration: 0.6,
          yoyo: true,
          repeat: 1,
        },
        '<'
      );
  }

  private createFlashElement(): HTMLElement {
    const flash = document.createElement('div');
    flash.className =
      'jackpot-effect absolute inset-0 bg-yellow-400/20 rounded-xl backdrop-blur-sm z-10';
    return flash;
  }

  private addCounterAnimation(
    timeline: gsap.core.Timeline,
    element: ElementRef,
    startValue: number,
    endValue: number
  ): void {
    const obj = { value: startValue };
    timeline.to(obj, {
      duration: this.calculateDuration(endValue - startValue),
      value: endValue,
      ease: 'power1.inOut',
      onUpdate: () => this.updateCounter(obj.value, endValue, element),
      onComplete: () => this.resetElementStyles(element, endValue),
    });
  }

  private updateCounter(currentValue: number, endValue: number, element: ElementRef): void {
    const progress = currentValue / endValue;
    const fluctuation = Math.random() * (100 * (1 - progress)) - 50 * (1 - progress);
    const displayValue = Math.floor(currentValue + fluctuation);
    element.nativeElement.textContent = '€' + displayValue.toLocaleString();
  }

  private resetElementStyles(element: ElementRef, finalValue: number): void {
    element.nativeElement.textContent = '€' + finalValue.toLocaleString();
    element.nativeElement.style.color = '';
    element.nativeElement.style.textShadow = '';
    element.nativeElement.style.transform = '';
  }

  private calculateDuration(increase: number): number {
    if (increase > this.MEGA_BONUS_THRESHOLD) return this.ANIMATION_DURATIONS.MEGA;
    if (increase > this.BONUS_THRESHOLD) return this.ANIMATION_DURATIONS.BONUS;
    return this.ANIMATION_DURATIONS.BASE;
  }

  private addMegaBonusEffect(element: ElementRef): void {
    const effectContainer = this.createEffectContainer(element);
    this.addGlowEffect(effectContainer, true);
    this.addFloatingSymbols(element, effectContainer, 10, 50);
  }

  private addBonusEffect(element: ElementRef): void {
    const effectContainer = this.createEffectContainer(element);
    this.addGlowEffect(effectContainer, false);
    this.addFloatingSymbols(element, effectContainer, 5, 30);
  }

  private createEffectContainer(element: ElementRef): HTMLElement {
    const container = element.nativeElement.parentElement;
    const effectContainer = document.createElement('div');
    effectContainer.className =
      'jackpot-effect absolute inset-0 pointer-events-none overflow-hidden z-20';
    container.appendChild(effectContainer);
    return effectContainer;
  }

  private addGlowEffect(container: HTMLElement, isMega: boolean): void {
    const config = isMega
      ? {
          shadow: '0 0 30px rgba(255,215,0,0.8), 0 0 60px rgba(255,165,0,0.6)',
          scale: 1.1,
          duration: 1.5,
          repeat: 2,
        }
      : {
          shadow: '0 0 20px rgba(255,215,0,0.4)',
          scale: 1.05,
          duration: 0.8,
          repeat: 1,
        };

    gsap.to(container, {
      boxShadow: config.shadow,
      scale: config.scale,
      opacity: 0,
      duration: config.duration,
      repeat: config.repeat,
      ease: 'power2.inOut',
      onComplete: () => container.remove(),
    });
  }

  private addFloatingSymbols(
    element: ElementRef,
    container: HTMLElement,
    count: number,
    radius: number
  ): void {
    const rect = element.nativeElement.getBoundingClientRect();
    const centerX = rect.width / 2;
    const centerY = rect.height / 2;
    const symbols = Object.values(this.SYMBOLS);

    Array.from({ length: count }).forEach((_, i) => {
      const symbol = this.createSymbol(symbols, container);
      const angle = (i / count) * Math.PI * 2;
      this.animateSymbol(symbol, centerX, centerY, angle, radius);
    });
  }

  private createSymbol(symbols: string[], container: HTMLElement): HTMLElement {
    const symbol = document.createElement('div');
    symbol.textContent = symbols[Math.floor(Math.random() * symbols.length)];
    symbol.className = 'jackpot-effect absolute text-2xl';
    container.appendChild(symbol);
    return symbol;
  }

  private animateSymbol(
    symbol: HTMLElement,
    centerX: number,
    centerY: number,
    angle: number,
    radius: number
  ): void {
    gsap.fromTo(
      symbol,
      {
        x: centerX,
        y: centerY,
        opacity: 0,
        scale: 0,
      },
      {
        x: centerX + Math.cos(angle) * radius,
        y: centerY + Math.sin(angle) * radius,
        opacity: 1,
        scale: 1,
        duration: 1.5,
        ease: 'back.out(1.2)',
        onComplete: () => this.fadeOutSymbol(symbol),
      }
    );
  }

  private fadeOutSymbol(symbol: HTMLElement): void {
    gsap.to(symbol, {
      opacity: 0,
      scale: 0,
      duration: 0.5,
      onComplete: () => symbol.remove(),
    });
  }
}
